@@ -464,13 +464,18 @@ class Input(ScrollView):
     def _cursor_offset(self) -> int:
         """The cell offset of the cursor."""
         offset = self._position_to_cell(self.cursor_position)
-        if self._cursor_at_end:
+        if self.cursor_at_end:
             offset += 1
         return offset
 
     @property
-    def _cursor_at_end(self) -> bool:
-        """Flag to indicate if the cursor is at the end"""
+    def cursor_at_start(self) -> bool:
+        """Flag to indicate if the cursor is at the start."""
+        return self.cursor_position == 0
+
+    @property
+    def cursor_at_end(self) -> bool:
+        """Flag to indicate if the cursor is at the end."""
         return self.cursor_position == len(self.value)
 
     def check_consume_key(self, key: str, character: str | None) -> bool:
@@ -513,7 +518,7 @@ class Input(ScrollView):
 
     @property
     def cursor_screen_offset(self) -> Offset:
-        """The offset of the cursor of this input in screen-space. (x, y)/(column, row)"""
+        """The offset of the cursor of this input in screen-space. (x, y)/(column, row)."""
         x, y, _width, _height = self.content_region
         scroll_x, _ = self.scroll_offset
         return Offset(x + self._cursor_offset - scroll_x, y)
@@ -536,6 +541,10 @@ class Input(ScrollView):
         if self._initial_value:
             self.cursor_position = len(self.value)
             self._initial_value = False
+        else:
+            # Force a re-validation of the selection to ensure it accounts for
+            # the length of the new value
+            self.selection = self.selection
 
     def _watch_valid_empty(self) -> None:
         """Repeat validation when valid_empty changes."""
@@ -636,7 +645,7 @@ class Input(ScrollView):
                 if self._cursor_visible:
                     cursor_style = self.get_component_rich_style("input--cursor")
                     cursor = self.cursor_position
-                    if not show_suggestion and self._cursor_at_end:
+                    if not show_suggestion and self.cursor_at_end:
                         result.pad_right(1)
                     result.stylize(cursor_style, cursor, cursor + 1)
 
@@ -760,12 +769,19 @@ class Input(ScrollView):
         self._selecting = True
         self.capture_mouse()
 
-    async def _on_mouse_up(self, event: events.MouseUp) -> None:
+    def _end_selecting(self) -> None:
+        """End selecting if it is currently active."""
         if self._selecting:
             self._selecting = False
             self.release_mouse()
             self._restart_blink()
 
+    async def _on_mouse_release(self, _event: events.MouseRelease) -> None:
+        self._end_selecting()
+
+    async def _on_mouse_up(self, _event: events.MouseUp) -> None:
+        self._end_selecting()
+
     async def _on_mouse_move(self, event: events.MouseMove) -> None:
         if self._selecting:
             # As we drag the mouse, we update the end position of the selection,
@@ -844,7 +860,7 @@ class Input(ScrollView):
         if select:
             self.selection = Selection(start, end + 1)
         else:
-            if self._cursor_at_end and self._suggestion:
+            if self.cursor_at_end and self._suggestion:
                 self.value = self._suggestion
                 self.cursor_position = len(self.value)
             else:
